/*

*/

import renderer.scene.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
<pre>{@code
                   y
                   |
                   |
                   | v[4]
                 1 +-----------------+ v[5]=(1,1,0)
                  /|                /|
                /  |              /  |
              /    |            /    |
            /      |          /      |
      v[7] +-----------------+ v[6]  |
           |       |         |       |                    y
           |       |         |       |                    |
           |       |         |       | v[1]               |
           |  v[0] +---------|-------+------> x           |
           |      /          |      /1                    |
           |    /            |    /                       +-----> x
           |  /              |  /                        /
           |/                |/                         /
         1 +-----------------+                         /
         / v[3]=(0,0,1)      v[2]=(1,0,1)            z
       /
     /
    z
}</pre>
*/
public class OfflineCube
{
   public static void main(String[] args)
   {
      // Create a Model object to hold the geometry.
      Model model = new Model();

      // Add geometry to the Model.
      Vertex v0 = new Vertex(0.0, 0.0, 0.0);  // four vertices around the bottom face
      Vertex v1 = new Vertex(1.0, 0.0, 0.0);
      Vertex v2 = new Vertex(1.0, 0.0, 1.0);
      Vertex v3 = new Vertex(0.0, 0.0, 1.0);
      Vertex v4 = new Vertex(0.0, 1.0, 0.0);  // four vertices around the top face
      Vertex v5 = new Vertex(1.0, 1.0, 0.0);
      Vertex v6 = new Vertex(1.0, 1.0, 1.0);
      Vertex v7 = new Vertex(0.0, 1.0, 1.0);

      model.addLineSegment(v0, v1);  // bottom face
      model.addLineSegment(v1, v2);  // bottom face
      model.addLineSegment(v2, v3);  // bottom face
      model.addLineSegment(v3, v0);  // bottom face
      model.addLineSegment(v4, v5);  // top face
      model.addLineSegment(v5, v6);  // top face
      model.addLineSegment(v6, v7);  // top face
      model.addLineSegment(v7, v4);  // top face
      model.addLineSegment(v0, v4);  // back face
      model.addLineSegment(v1, v5);  // back face
      model.addLineSegment(v3, v7);  // front face
      model.addLineSegment(v2, v6);  // front face


      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Add the Model to the Scene.
      scene.addModel(model);

      // Push the model away from where the camera is.
      for (Vertex v : model.vertexList)
      {
         v.z -= 2;
      }

      // Log interesting information to standard output.
      //System.out.println( scene.toString() );

      // Create a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;
      FrameBuffer fb = new FrameBuffer(width, height);

      for (int i = 0; i <= 100; i++)
      {
         // Render again.
         fb.clearFB(Color.darkGray);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_OfflineCube_Frame%03d.ppm", i));

         // Update the scene by translating in
         // camera space each of the 8 vertices.
         for (Vertex v : model.vertexList)
         {
            v.x += 0.1;
          //v.z -= 0.01;
         }
      }
   }
}
